#!/usr/bin/env python3

import argparse
import os
import struct

def create_header(args, size):
	header = struct.pack('32s32s32s32s32s',
		args.part_name.encode('ascii'),
		str(size).encode('ascii'),
		args.part_version.encode('ascii'),
		"".encode('ascii'),
		args.rootfs_version.encode('ascii'))

	return header

def create_output(args):
	in_st = os.stat(args.input_file)
	in_size = in_st.st_size

	header = create_header(args, in_size)
	print(header)

	in_f = open(args.input_file, 'r+b')
	in_bytes = in_f.read(in_size)
	in_f.close()

	out_f = open(args.output_file, 'w+b')
	out_f.write(header)
	out_f.write(in_bytes)
	out_f.close()

def main():
	global args

	parser = argparse.ArgumentParser(description='')

	parser.add_argument('--input-file',
		dest='input_file',
		action='store',
		type=str,
		help='Input file')

	parser.add_argument('--output-file',
		dest='output_file',
		action='store',
		type=str,
		help='Output file')

	parser.add_argument('--part-name',
		dest='part_name',
		action='store',
		type=str,
		help='Partition Name')

	parser.add_argument('--part-version',
		dest='part_version',
		action='store',
		type=str,
		help='Partition Version')

	parser.add_argument('--rootfs-version',
		dest='rootfs_version',
		action='store',
		type=str,
		help='RootFS lib version')

	args = parser.parse_args()

	if not args.rootfs_version:
		args.rootfs_version = ""

	if ((not args.input_file) or
	    (not args.output_file) or
	    (not args.part_name) or
	    (not args.part_version)):
		parser.print_help()

	create_output(args)

main()



#!/usr/bin/env python3

from os import getenv, environ
from pathlib import Path
from subprocess import run, PIPE
from sys import argv
import json

if len(argv) != 2:
    print("JSON info files script requires output file as argument")
    exit(1)

output_path = Path(argv[1])

assert getenv("WORK_DIR"), "$WORK_DIR required"

work_dir = Path(getenv("WORK_DIR"))

output = {}


def get_initial_output(image_info):
    # preserve existing profiles.json
    if output_path.is_file():
        profiles = json.loads(output_path.read_text())
        if profiles["version_code"] == image_info["version_code"]:
            return profiles
    return image_info


for json_file in work_dir.glob("*.json"):
    image_info = json.loads(json_file.read_text())

    if not output:
        output = get_initial_output(image_info)

    # get first and only profile in json file
    device_id, profile = next(iter(image_info["profiles"].items()))
    if device_id not in output["profiles"]:
        output["profiles"][device_id] = profile
    else:
        output["profiles"][device_id]["images"].extend(profile["images"])

# make image lists unique by name, keep last/latest
for device_id, profile in output.get("profiles", {}).items():
    profile["images"] = list({e["name"]: e for e in profile["images"]}.values())


if output:
    (
        default_packages,
        output["arch_packages"],
        linux_version,
        linux_release,
        linux_vermagic,
    ) = run(
        [
            "make",
            "--no-print-directory",
            "-C",
            "target/linux/",
            "val.DEFAULT_PACKAGES",
            "val.ARCH_PACKAGES",
            "val.LINUX_VERSION",
            "val.LINUX_RELEASE",
            "val.LINUX_VERMAGIC",
            "V=s",
        ],
        stdout=PIPE,
        check=True,
        env=environ.copy().update({"TOPDIR": Path().cwd()}),
        universal_newlines=True,
    ).stdout.splitlines()

    output["default_packages"] = sorted(default_packages.split())
    output["linux_kernel"] = {
        "version": linux_version,
        "release": linux_release,
        "vermagic": linux_vermagic,
    }
    output_path.write_text(json.dumps(output, sort_keys=True, separators=(",", ":")))
else:
    print("JSON info file script could not find any JSON files for target")



#!/usr/bin/env python3

"""
Whole Flash Image Tag

{
	u32 crc32;
	u32 version;
	u32 chipID;
	u32 flashType;
	u32 flags;
}

CRC32: Ethernet (Poly 0x04C11DB7)

Version:
	0x00005700: Any version
	0x00005731: NAND 1MB data partition
	0x00005732: Normal version

Chip ID:
	Broadcom Chip ID
	0x00006328: BCM6328
	0x00006362: BCM6362
	0x00006368: BCM6368
	0x00063268: BCM63268

Flash Type:
	1: NOR
	2: NAND 16k blocks
	3: NAND 128k blocks
	4: NAND 256k blocks
	5: NAND 512k blocks
	6: NAND 1MB blocks
	7: NAND 2MB blocks

Flags:
	0x00000001: PMC
	0x00000002: Secure BootROM

"""

import argparse
import os
import struct
import binascii


def auto_int(x):
    return int(x, 0)


def create_tag(args, in_bytes):
    # JAM CRC32 is bitwise not and unsigned
    crc = ~binascii.crc32(in_bytes) & 0xFFFFFFFF
    tag = struct.pack(
        ">IIIII",
        crc,
        args.tag_version,
        args.chip_id,
        args.flash_type,
        args.flags,
    )
    return tag


def create_output(args):
    in_st = os.stat(args.input_file)
    in_size = in_st.st_size

    in_f = open(args.input_file, "r+b")
    in_bytes = in_f.read(in_size)
    in_f.close()

    tag = create_tag(args, in_bytes)

    out_f = open(args.output_file, "w+b")
    out_f.write(in_bytes)
    out_f.write(tag)
    out_f.close()


def main():
    global args

    parser = argparse.ArgumentParser(description="")

    parser.add_argument(
        "--input-file",
        dest="input_file",
        action="store",
        type=str,
        help="Input file",
    )

    parser.add_argument(
        "--output-file",
        dest="output_file",
        action="store",
        type=str,
        help="Output file",
    )

    parser.add_argument(
        "--version",
        dest="tag_version",
        action="store",
        type=auto_int,
        help="WFI Tag Version",
    )

    parser.add_argument(
        "--chip-id",
        dest="chip_id",
        action="store",
        type=auto_int,
        help="WFI Chip ID",
    )

    parser.add_argument(
        "--flash-type",
        dest="flash_type",
        action="store",
        type=auto_int,
        help="WFI Flash Type",
    )

    parser.add_argument(
        "--flags", dest="flags", action="store", type=auto_int, help="WFI Flags"
    )

    args = parser.parse_args()

    if not args.flags:
        args.flags = 0

    if (
        (not args.input_file)
        or (not args.output_file)
        or (not args.tag_version)
        or (not args.chip_id)
        or (not args.flash_type)
    ):
        parser.print_help()
    else:
        create_output(args)


main()



#!/usr/bin/env python3

import argparse
import binascii
import hashlib
import os
import struct

def create_header(key, version, iv, random, size):
	header = struct.pack('32s32s32s32s32s', key, version, iv, random, size)

	return header

def create_output(args):
	in_st = os.stat(args.input_file)
	in_size = in_st.st_size

	key = "".encode('ascii')
	version = args.version.encode('ascii')
	iv = "".encode('ascii')
	random = "".encode('ascii')
	size = str(in_size).encode('ascii')
	header = create_header(key, version, iv, random, size)

	out_f = open(args.output_file, 'w+b')
	out_f.write(header)
	out_f.close()

	md5 = hashlib.md5()
	md5.update(header[0x60:0x80])
	md5.update(header[0x20:0x40])
	md5_1 = md5.digest()

	md5 = hashlib.md5()
	md5.update(header[0x80:0xA0])
	md5.update(header[0x20:0x40])
	md5_2 = md5.digest()

	key = md5_1 + md5_2

	key_f = open(args.key_file, 'w+b')
	key_f.write(binascii.hexlify(bytearray(key)))
	key_f.close()

	print("AES 256 CBC Key:", binascii.hexlify(bytearray(key)))

def main():
	global args

	parser = argparse.ArgumentParser(description='')

	parser.add_argument('--input-file',
		dest='input_file',
		action='store',
		type=str,
		help='Input file')

	parser.add_argument('--key-file',
		dest='key_file',
		action='store',
		type=str,
		help='AES 256 CBC Key File')

	parser.add_argument('--output-file',
		dest='output_file',
		action='store',
		type=str,
		help='Output file')

	parser.add_argument('--version',
		dest='version',
		action='store',
		type=str,
		help='Version')

	args = parser.parse_args()

	if ((not args.input_file) or
	    (not args.key_file) or
	    (not args.output_file) or
	    (not args.version)):
		parser.print_help()

	create_output(args)

main()



#!/usr/bin/env python3
#
# b43 firmware file squasher
# Removes unnecessary firmware files
#
# Copyright (c) 2009 Michael Buesch <mb@bu3sch.de>
#
# Licensed under the GNU/GPL version 2 or (at your option) any later version.
#

import sys
import os

def usage():
	print("Usage: %s PHYTYPES COREREVS /path/to/extracted/firmware" % sys.argv[0])
	print("")
	print("PHYTYPES is a comma separated list of:")
	print("A         => A-PHY")
	print("AG        => Dual A-PHY G-PHY")
	print("G         => G-PHY")
	print("LP        => LP-PHY")
	print("N         => N-PHY")
	print("HT        => HT-PHY")
	print("LCN       => LCN-PHY")
	print("LCN40     => LCN40-PHY")
	print("AC        => AC-PHY")
	print("")
	print("COREREVS is a comma separated list of core revision numbers.")

if len(sys.argv) != 4:
	usage()
	sys.exit(1)

phytypes = sys.argv[1]
corerevs = sys.argv[2]
fwpath = sys.argv[3]

phytypes = phytypes.split(',')
try:
	corerevs = [int(r) for r in corerevs.split(',')]
except ValueError:
	print("ERROR: \"%s\" is not a valid COREREVS string\n" % corerevs)
	usage()
	sys.exit(1)


fwfiles = os.listdir(fwpath)
fwfiles = [str for str in fwfiles if str.endswith(".fw")]
if not fwfiles:
	print("ERROR: No firmware files found in %s" % fwpath)
	sys.exit(1)

required_fwfiles = []

def revs_match(revs_a, revs_b):
	for rev in revs_a:
		if rev in revs_b:
			return True
	return False

def phytypes_match(types_a, types_b):
	for type in types_a:
		type = type.strip().upper()
		if type in types_b:
			return True
	return False

revmapping = {
	"ucode2.fw"		: ( (2,3,),		("G",), ),
	"ucode4.fw"		: ( (4,),		("G",), ),
	"ucode5.fw"		: ( (5,6,7,8,9,10,),	("G","A","AG",), ),
	"ucode11.fw"		: ( (11,12,),		("N",), ),
	"ucode13.fw"		: ( (13,),		("LP","G",), ),
	"ucode14.fw"		: ( (14,),		("LP",), ),
	"ucode15.fw"		: ( (15,),		("LP",), ),
	"ucode16_mimo.fw"	: ( (16,17,18,19,23,),	("N",), ),
#	"ucode16_lp.fw"		: ( (16,17,18,19,),	("LP",), ),
	"ucode24_lcn.fw"	: ( (24,),		("LCN",), ),
	"ucode25_mimo.fw"	: ( (25,28,),		("N",), ),
	"ucode25_lcn.fw"	: ( (25,28,),		("LCN",), ),
	"ucode26_mimo.fw"	: ( (26,),		("HT",), ),
	"ucode29_mimo.fw"	: ( (29,),		("HT",), ),
	"ucode30_mimo.fw"	: ( (30,),		("N",), ),
	"ucode33_lcn40.fw"	: ( (33,),		("LCN40",), ),
	"ucode40.fw"		: ( (40,),		("AC",), ),
	"ucode42.fw"		: ( (42,),		("AC",), ),
	"pcm4.fw"		: ( (1,2,3,4,),		("G",), ),
	"pcm5.fw"		: ( (5,6,7,8,9,10,),	("G","A","AG",), ),
}

initvalmapping = {
	"a0g1initvals5.fw"	: ( (5,6,7,8,9,10,),	("AG",), ),
	"a0g0initvals5.fw"	: ( (5,6,7,8,9,10,),	("A", "AG",), ),
	"b0g0initvals2.fw"	: ( (2,4,),		("G",), ),
	"b0g0initvals5.fw"	: ( (5,6,7,8,9,10,),	("G",), ),
	"b0g0initvals13.fw"	: ( (13,),		("G",), ),
	"n0initvals11.fw"	: ( (11,12,),		("N",), ),
	"n0initvals16.fw"	: ( (16,17,18,23,),	("N",), ),
	"n0initvals24.fw"	: ( (24,),		("N",), ),
	"n0initvals25.fw"	: ( (25,28,),		("N",), ),
	"n16initvals30.fw"	: ( (30,),		("N",), ),
	"lp0initvals13.fw"	: ( (13,),		("LP",), ),
	"lp0initvals14.fw"	: ( (14,),		("LP",), ),
	"lp0initvals15.fw"	: ( (15,),		("LP",), ),
#	"lp0initvals16.fw"	: ( (16,17,18,),	("LP",), ),
	"lcn0initvals24.fw"	: ( (24,),		("LCN",), ),
	"ht0initvals26.fw"	: ( (26,),		("HT",), ),
	"ht0initvals29.fw"	: ( (29,),		("HT",), ),
	"lcn400initvals33.fw"	: ( (33,),		("LCN40",), ),
	"ac0initvals40.fw"	: ( (40,),		("AC",), ),
	"ac1initvals42.fw"	: ( (42,),		("AC",), ),
	"a0g1bsinitvals5.fw"	: ( (5,6,7,8,9,10,),	("AG",), ),
	"a0g0bsinitvals5.fw"	: ( (5,6,7,8,9,10,),	("A", "AG"), ),
	"b0g0bsinitvals5.fw"	: ( (5,6,7,8,9,10,),	("G",), ),
	"n0bsinitvals11.fw"	: ( (11,12,),		("N",), ),
	"n0bsinitvals16.fw"	: ( (16,17,18,23,),	("N",), ),
	"n0bsinitvals24.fw"	: ( (24,),		("N",), ),
	"n0bsinitvals25.fw"	: ( (25,28,),		("N",), ),
	"n16bsinitvals30.fw"	: ( (30,),		("N",), ),
	"lp0bsinitvals13.fw"	: ( (13,),		("LP",), ),
	"lp0bsinitvals14.fw"	: ( (14,),		("LP",), ),
	"lp0bsinitvals15.fw"	: ( (15,),		("LP",), ),
#	"lp0bsinitvals16.fw"	: ( (16,17,18,),	("LP",), ),
	"lcn0bsinitvals24.fw"	: ( (24,),		("LCN",), ),
	"ht0bsinitvals26.fw"	: ( (26,),		("HT",), ),
	"ht0bsinitvals29.fw"	: ( (29,),		("HT",), ),
	"lcn400bsinitvals33.fw"	: ( (33,),		("LCN40",), ),
	"ac0bsinitvals40.fw"	: ( (40,),		("AC",), ),
	"ac1bsinitvals42.fw"	: ( (42,),		("AC",), ),
}

for f in fwfiles:
	if f in revmapping:
		if revs_match(corerevs, revmapping[f][0]) and\
		   phytypes_match(phytypes, revmapping[f][1]):
			required_fwfiles += [f]
		continue
	if f in initvalmapping:
		if revs_match(corerevs, initvalmapping[f][0]) and\
		   phytypes_match(phytypes, initvalmapping[f][1]):
			required_fwfiles += [f]
		continue
	print("WARNING: Firmware file %s not found in the mapping lists" % f)

for f in fwfiles:
	if f not in required_fwfiles:
		print("Deleting %s" % f)
		os.unlink(fwpath + '/' + f)




#!/usr/bin/env python3
"""
# SPDX-License-Identifier: GPL-2.0-or-later
#
# sercomm-pid.py: Creates Sercomm device PID
#
# Copyright © 2022 Mikhail Zhilkin
"""

import argparse
import binascii
import struct

PID_SIZE	= 0x70
PADDING		= 0x30
PADDING_TAIL	= 0x0

def auto_int(x):
	return int(x, 0)

def create_pid_file(args):
	pid_file = open(args.pid_file, "wb")
	buf = get_pid(args)
	pid_file.write(buf)
	pid_file.close()

def get_pid(args):
	buf = bytearray([PADDING] * PID_SIZE)

	if not args.hw_id:
		enc = args.hw_version.rjust(14, '0').encode('ascii')
		struct.pack_into('>14s', buf, 0x0, enc)
	else:
		enc = args.hw_version.rjust(8, '0').encode('ascii')
		struct.pack_into('>8s', buf, 0x0, enc)

		enc = binascii.hexlify(args.hw_id.encode()).upper()
		struct.pack_into('>6s', buf, 0x8, enc)

	enc = args.sw_version.rjust(4, '0').encode('ascii')
	struct.pack_into('>4s', buf, 0x64, enc)

	if (args.extra_padd_size):
		tail = bytearray([PADDING_TAIL] * args.extra_padd_size)
		if (args.extra_padd_byte):
			struct.pack_into ('<i', tail, 0x0,
					  args.extra_padd_byte)
		elif not args.hw_id:
			tail[0] = 0x0D
			tail[1] = 0x0A
		buf += tail

	return buf

def main():
	global args

	parser = argparse.ArgumentParser(description='This script \
		generates firmware PID for the Sercomm-based devices')

	parser.add_argument('--hw-version',
		dest='hw_version',
		action='store',
		type=str,
		help='Sercomm hardware version')

	parser.add_argument('--hw-id',
		dest='hw_id',
		action='store',
		type=str,
		help='Sercomm hardware ID')

	parser.add_argument('--sw-version',
		dest='sw_version',
		action='store',
		type=str,
		help='Sercomm software version')

	parser.add_argument('--pid-file',
		dest='pid_file',
		action='store',
		type=str,
		help='Output PID file')

	parser.add_argument('--extra-padding-size',
		dest='extra_padd_size',
		action='store',
		type=auto_int,
		help='Size of extra NULL padding at the end of the file \
			(optional)')

	parser.add_argument('--extra-padding-first-byte',
		dest='extra_padd_byte',
		action='store',
		type=auto_int,
		help='First byte of extra padding (optional)')

	args = parser.parse_args()

	if ((not args.hw_version) or
	    (not args.sw_version) or 
	    (not args.pid_file)):
		parser.print_help()
		exit()

	create_pid_file(args)

main()



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-or-later

import argparse
import struct

from binascii import crc32
from dataclasses import dataclass
from itertools import cycle
from typing import List


def xor(data: bytes) -> bytes:
    passphrase = "Seek AGREEMENT for the date of completion.\0"
    pw = cycle(bytearray(passphrase.encode('ascii')))
    return bytearray(b ^ next(pw) for b in data)


def add_fw_header(data: bytes, magic: int, hwid: int, build_id: int,
                  offsets: List[int]) -> bytes:
    unknown_1 = 0x01
    unknown_2 = 0x0000
    unknown_3 = 0x00000000
    unknown_4 = 0x01000000
    file_crc = crc(data, 0)

    header_struct = struct.Struct('>QIBBHIIIIII' + 'I' * len(offsets))
    header_size = header_struct.size
    file_size = header_size + len(data)

    header_offsets = map(lambda x: x + header_size, offsets)

    header_data = header_struct.pack(magic, file_size, unknown_1, len(offsets),
                                     unknown_2, hwid, build_id, unknown_3,
                                     build_id, unknown_4, *header_offsets,
                                     file_crc)
    return header_data + data


def add_file_header(data: bytes, filename: str, build_id: int) -> bytes:
    unknown1 = 0x01000000
    unknown2 = 0x00000000
    file_crc = crc(data, 0)

    header_struct = struct.Struct(">16sIIIII")
    file_size = header_struct.size + len(data)

    header_data = header_struct.pack(filename.encode('ascii'), file_size,
                                     unknown1, build_id, unknown2, file_crc)
    return header_data + data


def crc(data: bytes, init_val: int) -> int:
    return 0xffffffff ^ (crc32(data, 0xffffffff ^ init_val))


@dataclass
class Partition:
    name: str
    size: int


def main():
    partitions = [
        Partition(name='kernel', size=2048 * 1024),
        Partition(name='root', size=9216 * 1024),
        Partition(name='userdisk', size=3076 * 1024),
    ]

    parser = argparse.ArgumentParser(prog='moxa-encode-fw',
                                     description='MOXA IW firmware encoder')
    parser.add_argument('-i', '--input', required=True, type=str, help='Firmware file')
    parser.add_argument('-o', '--output', required=True, type=str, help="Output path for encoded firmware file")
    parser.add_argument('-m', '--magic', required=True, type=lambda x: int(x,0), help="Magic for firmware header")
    parser.add_argument('-d', '--hwid', required=True, type=lambda x: int(x,0), help="Hardware id of device")
    parser.add_argument('-b', '--buildid', required=True, type=lambda x: int(x,0), help="Build id of firmware")
    args = parser.parse_args()

    with open(args.input, 'rb') as input_file:
        firmware = bytearray(input_file.read())

    offsets = []
    pos_input = 0
    pos_output = 0
    firmware_seg = bytearray()

    for partition in partitions:
        part_data = firmware[pos_input:pos_input + partition.size]

        # just to make sure that no partition is empty
        if len(part_data) == 0:
            part_data = bytearray([0x00])

        header = add_file_header(part_data, partition.name, args.buildid)
        firmware_seg += header

        offsets.append(pos_output)
        pos_input += partition.size
        pos_output += len(header)

    moxa_firmware = add_fw_header(firmware_seg, args.magic, args.hwid, args.buildid, offsets)

    encrypted = xor(moxa_firmware)
    with open(args.output, 'wb') as output_file:
        output_file.write(encrypted)


if __name__ == '__main__':
    main()



#!/usr/bin/python3
# SPDX-License-Identifier: GPL-2.0-or-later
#
# Copyright (C) 2022 OpenWrt.org
#
# ./cameo-tag.py <uImageFileName> <OffsetOfRootFS>
#
# CAMEO tag generator used for the D-Link DGS-1210 switches. Their U-Boot
# loader checks for the string CAMEOTAG and a checksum in the kernel and
# rootfs partitions. If not found it complains about the boot image.
# Nevertheless it will boot if the tags are available in the secondary
# boot partitions. If some day we want to overwrite the original vendor
# partition we must have the tags in place. To solve this we insert the
# tag two times into the kernel image.
#
# To understand what we do here it is helpful to explain how the original
# CAMEO tag generation/checking works. The firmware consists of two parts.
# A kernel uImage (<1.5MB) and a rootfs image (<12MB) that are written to
# their respective mtd partitions. The default generator simply checksums
# both parts and appends 16 bytes [<CAMEOTAG><0001><checksum>] to each part.
# The checksum is only an addition of all preceding bytes (b0+b1+b2+...).
# A tag does not interfere with any data in the images itself. During boot
# the loader will scan all primary/secondary partitions (2*kernel, 2*rootfs)
# until it finds the CAMEO tag. If checksums match everything is fine.
# If all 4 fail we are lost. Luckily the loader does not care about where
# the tags are located and ignores any data beyond a tag.
#
# The OpenWrt image consists of a kernel (>1.5MB) and a rootfs. There is
# no chance to add CAMEO tags at the default locations, since the kernel spans
# both the original kernel partition and the start of the rootfs partition.
# This would leave the kernel partition without a tag. So we must find suitable
# space.
#
# Location for original kernel partition is at the end of the uImage header.
# We will reuse the last bytes of the IH_NAME field. This is the tricky part
# because we have the header CRC and the CAMEO checksum that must match the
# whole header. uImage header CRC checksums all data except the CRC itself. The
# for CAMEO checksum in turn, checksums all preceding data except itself.
# Changing one of both results in a change of the other, but data trailing the
# CAMEO checksum only influences the CRC.
#
# Location for original rootfs partition is very simple. It is behind the
# OpenWrt compressed kernel image file that spans into the rootfs. So
# the tag will be written somewhere to the following rootfs partition and
# can be found by U-Boot. The CAMEO checksum calculation must start at the
# offset of the original rootfs partition and includes the "second" half of the
# "split" kernel uImage.

import argparse
import os
import zlib

READ_UNTIL_EOF = -1
UIMAGE_HEADER_SIZE = 64
UIMAGE_CRC_OFF = 4
UIMAGE_CRC_END = 8
UIMAGE_NAME_OFF = 32
UIMAGE_NAME_END = 56
UIMAGE_SUM_OFF = 56
UIMAGE_SUM_END = 60
UIMAGE_INV_OFF = 60
UIMAGE_INV_END = 64
CAMEO_TAG = bytes([0x43, 0x41, 0x4d, 0x45, 0x4f, 0x54, 0x41, 0x47, 0x00, 0x00, 0x00, 0x01])
IMAGE_NAME = bytes([0x4f, 0x70, 0x65, 0x6e, 0x57, 0x72, 0x74, 0x00, 0x00, 0x00, 0x00, 0x00])
CRC_00 = bytes([0x00] * 4)
CRC_FF = bytes([0xff] * 4)

def read_buffer(offset, count):
    args.uimage_file.seek(offset)
    return bytearray(args.uimage_file.read(count))

def write_buffer(whence, buf):
    args.uimage_file.seek(0, whence)
    args.uimage_file.write(buf)

def cameosum(buf):
    return (sum(buf) & 0xffffffff).to_bytes(4, 'big')

def invertcrc(buf):
    return (zlib.crc32(buf) ^ 0xffffffff).to_bytes(4, 'little')

def checksum_header(buf):
    # To efficently get a combination, we will make use of the following fact:
    #     crc32(data + littleendian(crc32(data) ^ 0xffffffff)) = 0xffffffff
    #
    # After manipulation the uImage header looks like this:
    #     [...<ffffffff>...<OpenWrt><000000><CAMEOTAG><0001><checksum><InvCRC>]
    buf[UIMAGE_NAME_OFF:UIMAGE_NAME_END] = IMAGE_NAME + CAMEO_TAG
    buf[UIMAGE_CRC_OFF:UIMAGE_CRC_END] = CRC_FF
    buf[UIMAGE_SUM_OFF:UIMAGE_SUM_END] = cameosum(buf[0:UIMAGE_NAME_END])
    buf[UIMAGE_CRC_OFF:UIMAGE_CRC_END] = CRC_00
    buf[UIMAGE_INV_OFF:UIMAGE_INV_END] = invertcrc(buf[0:UIMAGE_SUM_END])
    buf[UIMAGE_CRC_OFF:UIMAGE_CRC_END] = CRC_FF
    return buf

parser = argparse.ArgumentParser(description='Insert CAMEO firmware tags.')
parser.add_argument('uimage_file', type=argparse.FileType('r+b'))
parser.add_argument('rootfs_start', type=int)
args = parser.parse_args()

args.uimage_file.seek(0, os.SEEK_END)
if args.uimage_file.tell() <= args.rootfs_start:
    raise ValueError(f"uImage must be larger than {args.rootfs_start} bytes")

# tag for the uImage Header of 64 bytes inside the kernel
# partition. Read and mangle it so it contains a valid CAMEO tag
# and checksum that matches perfectly to the uImage header CRC.

buf = checksum_header(read_buffer(0, UIMAGE_HEADER_SIZE))
write_buffer(os.SEEK_SET, buf)

# tag for the second part of the kernel that resides in the
# vendor rootfs partition. For this we will add the CAMEO tag
# and the checksum to the end of the image.

buf = read_buffer(args.rootfs_start, READ_UNTIL_EOF)
write_buffer(os.SEEK_END, CAMEO_TAG + cameosum(buf + CAMEO_TAG))



#!/usr/bin/env python3

import argparse
import hashlib
import os

def create_output(args):
	in_st = os.stat(args.input_file)
	in_size = in_st.st_size

	in_f = open(args.input_file, 'r+b')
	in_bytes = in_f.read(in_size)
	in_f.close()

	if (args.pid_file):
		pid_st = os.stat(args.pid_file)
		pid_size = pid_st.st_size

		pid_f = open(args.pid_file, 'r+b')
		pid_bytes = pid_f.read(pid_size)
		pid_f.close()
	else:
		pid_bytes = bytes.fromhex(args.pid)

	sha256 = hashlib.sha256()
	sha256.update(in_bytes)

	out_f = open(args.output_file, 'w+b')
	out_f.write(pid_bytes)
	out_f.write(sha256.digest())
	out_f.write(in_bytes)
	out_f.close()

def main():
	global args

	parser = argparse.ArgumentParser(description='')

	parser.add_argument('--input-file',
		dest='input_file',
		action='store',
		type=str,
		help='Input file')

	parser.add_argument('--output-file',
		dest='output_file',
		action='store',
		type=str,
		help='Output file')

	parser.add_argument('--pid-file',
		dest='pid_file',
		action='store',
		type=str,
		help='Sercomm PID file')

	parser.add_argument('--pid',
		dest='pid',
		action='store',
		type=str,
		help='Sercomm PID')

	args = parser.parse_args()

	if ((not args.input_file) or
	    (not args.output_file) or
	    (not args.pid_file and not args.pid)):
		parser.print_help()

	create_output(args)

main()



#!/usr/bin/env python3

import argparse
import os
import struct

def auto_int(x):
	return int(x, 0)

def create_header(args, size):
	header = struct.pack('>III', args.entry_addr, args.load_addr, size)
	return header

def create_output(args):
	in_st = os.stat(args.input_file)
	in_size = in_st.st_size

	header = create_header(args, in_size)
	print(header)

	in_f = open(args.input_file, 'r+b')
	in_bytes = in_f.read(in_size)
	in_f.close()

	out_f = open(args.output_file, 'w+b')
	out_f.write(header)
	out_f.write(in_bytes)
	out_f.close()

def main():
	global args

	parser = argparse.ArgumentParser(description='')

	parser.add_argument('--entry-addr',
		dest='entry_addr',
		action='store',
		type=auto_int,
		help='Entry Address')

	parser.add_argument('--input-file',
		dest='input_file',
		action='store',
		type=str,
		help='Input file')

	parser.add_argument('--load-addr',
		dest='load_addr',
		action='store',
		type=auto_int,
		help='Load Address')

	parser.add_argument('--output-file',
		dest='output_file',
		action='store',
		type=str,
		help='Output file')

	args = parser.parse_args()

	if (not args.input_file) or (not args.output_file):
		parser.print_help()

	if not args.entry_addr:
		args.entry_addr = 0x80010000

	if not args.load_addr:
		args.load_addr = 0x80010000

	create_output(args)

main()



#!/usr/bin/env python3

"""
CFE Partition Tag

{
	u32 part_id;
	u32 part_size;
	u16 flags;
	char part_name[33];
	char part_version[21];
	u32 part_crc32;
}

"""

import argparse
import os
import struct
import binascii


PART_NAME_SIZE = 33
PART_VERSION_SIZE = 21


def auto_int(x):
    return int(x, 0)


def str_to_bytes_pad(string, size):
    str_bytes = string.encode()
    num_bytes = len(str_bytes)
    if num_bytes >= size:
        str_bytes = str_bytes[: size - 1] + "\0".encode()
    else:
        str_bytes += "\0".encode() * (size - num_bytes)
    return str_bytes


def create_tag(args, in_bytes, size):
    # JAM CRC32 is bitwise not and unsigned
    crc = ~binascii.crc32(in_bytes) & 0xFFFFFFFF

    tag = bytearray()
    tag += struct.pack(">I", args.part_id)
    tag += struct.pack(">I", size)
    tag += struct.pack(">H", args.part_flags)
    tag += str_to_bytes_pad(args.part_name, PART_NAME_SIZE)
    tag += str_to_bytes_pad(args.part_version, PART_VERSION_SIZE)
    tag += struct.pack(">I", crc)

    return tag


def create_output(args):
    in_st = os.stat(args.input_file)
    in_size = in_st.st_size

    in_f = open(args.input_file, "r+b")
    in_bytes = in_f.read(in_size)
    in_f.close()

    tag = create_tag(args, in_bytes, in_size)

    out_f = open(args.output_file, "w+b")
    out_f.write(tag)
    out_f.close()


def main():
    global args

    parser = argparse.ArgumentParser(description="")

    parser.add_argument(
        "--flags",
        dest="part_flags",
        action="store",
        type=auto_int,
        help="Partition Flags",
    )

    parser.add_argument(
        "--id",
        dest="part_id",
        action="store",
        type=auto_int,
        help="Partition ID",
    )

    parser.add_argument(
        "--input-file",
        dest="input_file",
        action="store",
        type=str,
        help="Input file",
    )

    parser.add_argument(
        "--output-file",
        dest="output_file",
        action="store",
        type=str,
        help="Output file",
    )

    parser.add_argument(
        "--name",
        dest="part_name",
        action="store",
        type=str,
        help="Partition Name",
    )

    parser.add_argument(
        "--version",
        dest="part_version",
        action="store",
        type=str,
        help="Partition Version",
    )

    args = parser.parse_args()

    if (
        (not args.part_flags)
        or (not args.part_id)
        or (not args.input_file)
        or (not args.output_file)
        or (not args.part_name)
        or (not args.part_version)
    ):
        parser.print_help()
    else:
        create_output(args)


main()



#!/usr/bin/env python3
"""
# OpenWrt download directory cleanup utility.
# Delete all but the very last version of the program tarballs.
#
# Copyright (C) 2010-2015 Michael Buesch <m@bues.ch>
# Copyright (C) 2013-2015 OpenWrt.org
"""

from __future__ import print_function

import sys
import os
import re
import getopt
import shutil

# Commandline options
opt_dryrun = False


def parseVer_1234(match, filepath):
    progname = match.group(1)
    progversion = (
        (int(match.group(2)) << 64)
        | (int(match.group(3)) << 48)
        | (int(match.group(4)) << 32)
        | (int(match.group(5)) << 16)
    )
    return (progname, progversion)


def parseVer_123(match, filepath):
    progname = match.group(1)
    try:
        patchlevel = match.group(5)
    except IndexError as e:
        patchlevel = None
    if patchlevel:
        patchlevel = ord(patchlevel[0])
    else:
        patchlevel = 0
    progversion = (
        (int(match.group(2)) << 64)
        | (int(match.group(3)) << 48)
        | (int(match.group(4)) << 32)
        | patchlevel
    )
    return (progname, progversion)


def parseVer_12(match, filepath):
    progname = match.group(1)
    try:
        patchlevel = match.group(4)
    except IndexError as e:
        patchlevel = None
    if patchlevel:
        patchlevel = ord(patchlevel[0])
    else:
        patchlevel = 0
    progversion = (int(match.group(2)) << 64) | (int(match.group(3)) << 48) | patchlevel
    return (progname, progversion)


def parseVer_r(match, filepath):
    progname = match.group(1)
    progversion = int(match.group(2)) << 64
    return (progname, progversion)


def parseVer_ymd_GIT_SHASUM(match, filepath):
    progname = match.group(1)
    progversion = (
        (int(match.group(2)) << 64)
        | (int(match.group(3)) << 48)
        | (int(match.group(4)) << 32)
    )
    return (progname, progversion)


def parseVer_ymd(match, filepath):
    progname = match.group(1)
    progversion = (
        (int(match.group(2)) << 64)
        | (int(match.group(3)) << 48)
        | (int(match.group(4)) << 32)
    )
    return (progname, progversion)


def parseVer_GIT(match, filepath):
    progname = match.group(1)
    st = os.stat(filepath)
    progversion = int(st.st_mtime) << 64
    return (progname, progversion)


extensions = (
    ".tar.gz",
    ".tar.bz2",
    ".tar.xz",
    ".tar.zst",
    ".orig.tar.gz",
    ".orig.tar.bz2",
    ".orig.tar.xz",
    ".zip",
    ".tgz",
    ".tbz",
    ".txz",
)

versionRegex = (
    (re.compile(r"(gcc[-_]\d+)\.(\d+)\.(\d+)"), parseVer_12),  # gcc.1.2
    (re.compile(r"(linux[-_]\d+\.\d+)\.(\d+)"), parseVer_r),  # linux.1
    (re.compile(r"(.+)[-_](\d+)\.(\d+)\.(\d+)\.(\d+)"), parseVer_1234),  # xxx-1.2.3.4
    (
        re.compile(r"(.+)[-_](\d\d\d\d)-?(\d\d)-?(\d\d)-"),
        parseVer_ymd_GIT_SHASUM,
    ),  # xxx-YYYY-MM-DD-GIT_SHASUM
    (re.compile(r"(.+)[-_](\d\d\d\d)-?(\d\d)-?(\d\d)"), parseVer_ymd),  # xxx-YYYY-MM-DD
    (re.compile(r"(.+)[-_]([0-9a-fA-F]{40,40})"), parseVer_GIT),  # xxx-GIT_SHASUM
    (re.compile(r"(.+)[-_](\d+)\.(\d+)\.(\d+)(\w?)"), parseVer_123),  # xxx-1.2.3a
    (re.compile(r"(.+)[-_]v(\d+)\.(\d+)\.(\d+)(\w?)"), parseVer_123),  # xxx-v1.2.3a
    (re.compile(r"(.+)[-_](\d+)_(\d+)_(\d+)"), parseVer_123),  # xxx-1_2_3
    (re.compile(r"(.+)[-_](\d+)\.(\d+)(\w?)"), parseVer_12),  # xxx-1.2a
    (re.compile(r"(.+)[-_]v(\d+)\.(\d+)(\w?)"), parseVer_12),  # xxx-v1.2a
    (re.compile(r"(.+)[-_]r?(\d+)"), parseVer_r),  # xxx-r1111
)

blacklist = [
    ("wl_apsta", re.compile(r"wl_apsta.*")),
    (".fw", re.compile(r".*\.fw")),
    (".arm", re.compile(r".*\.arm")),
    (".bin", re.compile(r".*\.bin")),
    ("rt-firmware", re.compile(r"RT[\d\w]+_Firmware.*")),
]


class EntryParseError(Exception):
    pass


class Entry:
    def __init__(self, directory, builddir, filename):
        self.directory = directory
        self.filename = filename
        self.builddir = builddir
        self.progname = ""
        self.fileext = ""
        self.filenoext = ""

        if os.path.isdir(self.getPath()):
            self.filenoext = filename
        else:
            for ext in extensions:
                if filename.endswith(ext):
                    filename = filename[0 : 0 - len(ext)]
                    self.filenoext = filename
                    self.fileext = ext
                    break
            else:
                print(self.filename, "has an unknown file-extension")
                raise EntryParseError("ext")
        for (regex, parseVersion) in versionRegex:
            match = regex.match(filename)
            if match:
                (self.progname, self.version) = parseVersion(
                    match, directory + "/" + filename + self.fileext
                )
                break
        else:
            print(self.filename, "has an unknown version pattern")
            raise EntryParseError("ver")

    def getPath(self):
        return (self.directory + "/" + self.filename).replace("//", "/")

    def getBuildPaths(self):
        paths = []
        for subdir in os.scandir(self.builddir):
            package_build_dir = os.path.join(subdir.path, self.filenoext)
            if os.path.exists(package_build_dir):
                paths.append(package_build_dir)
        return paths

    def deleteFile(self):
        path = self.getPath()
        print("Deleting", path)
        if not opt_dryrun:
            if os.path.isdir(path):
                shutil.rmtree(path)
            else:
                os.unlink(path)

    def deleteBuildDir(self):
        paths = self.getBuildPaths()
        for path in paths:
            print("Deleting BuildDir", path)
            if not opt_dryrun:
                    shutil.rmtree(path)

    def __ge__(self, y):
        return self.version >= y.version


def usage():
    print("OpenWrt download directory cleanup utility")
    print("Usage: " + sys.argv[0] + " [OPTIONS] <path/to/dl>")
    print("")
    print(" -d|--dry-run            Do a dry-run. Don't delete any files")
    print(" -B|--show-blacklist     Show the blacklist and exit")
    print(" -w|--whitelist ITEM     Remove ITEM from blacklist")
    print(
        " -D|--download-dir       Provide path to dl dir to clean also the build directory"
    )
    print(
        " -b|--build-dir          Provide path to build dir to clean also the build directory"
    )


def main(argv):
    global opt_dryrun

    try:
        (opts, args) = getopt.getopt(
            argv[1:],
            "hdBw:D:b:",
            [
                "help",
                "dry-run",
                "show-blacklist",
                "whitelist=",
                "download-dir=",
                "build-dir=",
            ],
        )
    except getopt.GetoptError as e:
        usage()
        return 1

    directory = "dl/"
    builddir = "build_dir/"

    for (o, v) in opts:
        if o in ("-h", "--help"):
            usage()
            return 0
        if o in ("-d", "--dry-run"):
            opt_dryrun = True
        if o in ("-w", "--whitelist"):
            for i in range(0, len(blacklist)):
                (name, regex) = blacklist[i]
                if name == v:
                    del blacklist[i]
                    break
            else:
                print("Whitelist error: Item", v, "is not in blacklist")
                return 1
        if o in ("-B", "--show-blacklist"):
            for (name, regex) in blacklist:
                sep = "\t\t"
                if len(name) >= 8:
                    sep = "\t"
                print("%s%s(%s)" % (name, sep, regex.pattern))
            return 0
        if o in ("-D", "--download-dir"):
            directory = v
        if o in ("-b", "--build-dir"):
            builddir = v

    if args:
        directory = args[0]

    if not os.path.exists(directory):
        print("Can't find download directory", directory)
        return 1

    if not os.path.exists(builddir):
        print("Can't find build directory", builddir)
        return 1

    # Create a directory listing and parse the file names.
    entries = []
    for filename in os.listdir(directory):
        if filename == "." or filename == "..":
            continue
        for (name, regex) in blacklist:
            if regex.match(filename):
                if opt_dryrun:
                    print(filename, "is blacklisted")
                break
        else:
            try:
                entries.append(Entry(directory, builddir, filename))
            except EntryParseError as e:
                pass

    # Create a map of programs
    progmap = {}
    for entry in entries:
        if entry.progname in progmap.keys():
            progmap[entry.progname].append(entry)
        else:
            progmap[entry.progname] = [
                entry,
            ]

    # Traverse the program map and delete everything but the last version
    for prog in progmap:
        lastVersion = None
        versions = progmap[prog]
        for version in versions:
            if lastVersion:
                if os.path.isdir(lastVersion.getPath()) and not os.path.isdir(version.getPath()):
                    continue
            if lastVersion is None or version >= lastVersion:
                lastVersion = version
        if lastVersion:
            for version in versions:
                if version is not lastVersion:
                    version.deleteFile()
                    if builddir:
                        version.deleteBuildDir()
            if opt_dryrun:
                print("Keeping", lastVersion.getPath())

    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/bin/env python3
"""
Parse the native package index files into a json file for use by downstream
tools.  See:

    https://github.com/openwrt/openwrt/commit/218ce40cd738f3373438aab82467807a8707fb9c

The "version 1" index.json contained ABI-versioned package names, making the
unusable by the ASU server.  The version 2 format contains package names that
have been stripped of their ABI version.
"""

import email.parser
import json


def parse_args():
    from argparse import ArgumentParser

    source_format = "apk", "opkg"

    parser = ArgumentParser()
    # fmt: off
    parser.add_argument("-a", "--architecture", required=True,
                        help="Required device architecture: like 'x86_64' or 'aarch64_generic'")
    parser.add_argument("-f", "--source-format", required=True, choices=source_format,
                        help="Required source format of input: 'apk' or 'opkg'")
    parser.add_argument(dest="source",
                        help="File name for input, '-' for stdin")
    # fmt: on
    args = parser.parse_args()
    return args


def parse_apk(text: str) -> dict:
    packages: dict = {}

    data = json.loads(text)
    for package in data.get("packages", []):
        package_name: str = package["name"]

        for tag in package.get("tags", []):
            if tag.startswith("openwrt:abiversion="):
                package_abi: str = tag.split("=")[-1]
                package_name = package_name.removesuffix(package_abi)
                break

        packages[package_name] = package["version"]

    return packages


def parse_opkg(text: str) -> dict:
    packages: dict = {}

    parser: email.parser.Parser = email.parser.Parser()
    chunks: list[str] = text.strip().split("\n\n")
    for chunk in chunks:
        package: dict = parser.parsestr(chunk, headersonly=True)
        package_name: str = package["Package"]
        if package_abi := package.get("ABIVersion"):
            package_name = package_name.removesuffix(package_abi)

        packages[package_name] = package["Version"]

    return packages


if __name__ == "__main__":
    import sys

    args = parse_args()

    input = sys.stdin if args.source == "-" else open(args.source, "r")
    with input:
        text: str = input.read()

    packages = parse_apk(text) if args.source_format == "apk" else parse_opkg(text)
    index = {
        "version": 2,
        "architecture": args.architecture,
        "packages": packages,
    }
    print(json.dumps(index, indent=2))



#!/usr/bin/env python3

import argparse
import re
import struct
import subprocess
import zlib


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--input-file', type=str, required=True)
    parser.add_argument('--output-file', type=str, required=True)
    parser.add_argument('--model', type=str, required=True)
    parser.add_argument('--region', type=str, required=True)
    parser.add_argument('--version', type=str, required=True)
    parser.add_argument('--hw-id-list', type=str)
    parser.add_argument('--model-list', type=str)
    parser.add_argument('--encryption-block-size', type=str, required=True)
    parser.add_argument('--openssl-bin', type=str, required=True)
    parser.add_argument('--key', type=str, required=True)
    parser.add_argument('--iv', type=str, required=True)
    args = parser.parse_args()

    assert re.match(r'V[0-9]\.[0-9]\.[0-9]\.[0-9]',
                    args.version), 'Version must start with Vx.x.x.x'
    encryption_block_size = int(args.encryption_block_size, 0)
    assert (encryption_block_size > 0 and encryption_block_size % 16 ==
            0), 'Encryption block size must be a multiple of the AES block size (16)'

    hw_id_list = args.hw_id_list.split(';') if args.hw_id_list else []
    model_list = args.model_list.split(';') if args.model_list else []
    hw_info = ';'.join(hw_id_list + model_list)

    image = open(args.input_file, 'rb').read()
    image_enc = []
    for i in range(0, len(image), encryption_block_size):
        chunk = image[i:i + encryption_block_size]
        chunk += b'\x00' * ((-len(chunk)) % 16)  # pad to AES block size (16)
        res = subprocess.run([
            args.openssl_bin,
            'enc',
            '-aes-256-cbc',
            '-nosalt',
            '-nopad',
            '-K', args.key,
            '-iv', args.iv
        ],
            check=True, input=chunk, stdout=subprocess.PIPE)
        image_enc.append(res.stdout)
    image_enc = b''.join(image_enc)

    image_with_header = struct.pack(
        '>32s32s64s64sIBBB13s200s100s12sII',
        args.model.encode('ascii'),
        args.region.encode('ascii'),
        args.version.encode('ascii'),
        b'Thu Jan 1 00:00:00 1970',  # static date for reproducibility
        0,  # product hw model
        0,  # model index
        len(hw_id_list),
        len(model_list),
        b'',  # reserved
        hw_info.encode('ascii'),
        b'',  # reserved
        b'encrpted_img',
        len(image_enc),
        encryption_block_size,
    ) + image_enc

    checksum = zlib.crc32(image_with_header, 0xffffffff) ^ 0xffffffff

    with open(args.output_file, 'wb') as outfile:
        outfile.write(image_with_header)
        outfile.write(struct.pack('>I', checksum))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

from os import getenv
from pathlib import Path
from sys import argv
import hashlib
import json

if len(argv) != 2:
    print("ERROR: JSON info script requires output arg")
    exit(1)

json_path = Path(argv[1])
file_path = Path(getenv("FILE_DIR")) / getenv("FILE_NAME")

if not file_path.is_file():
    print("Skip JSON creation for non existing file", file_path)
    exit(0)


def get_titles():
    titles = []
    for prefix in ["", "ALT0_", "ALT1_", "ALT2_", "ALT3_", "ALT4_", "ALT5_"]:
        title = {}
        for var in ["vendor", "model", "variant"]:
            if getenv("DEVICE_{}{}".format(prefix, var.upper())):
                title[var] = getenv("DEVICE_{}{}".format(prefix, var.upper()))

        if title:
            titles.append(title)

    if not titles:
        titles.append({"title": getenv("DEVICE_TITLE")})

    return titles


device_id = getenv("DEVICE_ID")

sha256_hash = hashlib.sha256()
with open(str(file_path),"rb") as f:
    # Read and update hash string value in blocks of 4K
    for byte_block in iter(lambda: f.read(4096),b""):
        sha256_hash.update(byte_block)

hash_file = sha256_hash.hexdigest()

if file_path.with_suffix(file_path.suffix + ".sha256sum").exists():
    hash_unsigned = (
        file_path.with_suffix(file_path.suffix + ".sha256sum").read_text().strip()
    )
else:
    hash_unsigned = hash_file

file_info = {
    "metadata_version": 1,
    "target": "{}/{}".format(getenv("TARGET"), getenv("SUBTARGET")),
    "version_code": getenv("VERSION_CODE"),
    "version_number": getenv("VERSION_NUMBER"),
    "source_date_epoch": int(getenv("SOURCE_DATE_EPOCH")),
    "profiles": {
        device_id: {
            "image_prefix": getenv("DEVICE_IMG_PREFIX"),
            "images": [
                {
                    "type": getenv("FILE_TYPE"),
                    "name": getenv("FILE_NAME"),
                    "sha256": hash_file,
                    "sha256_unsigned": hash_unsigned,
                }
            ],
            "device_packages": getenv("DEVICE_PACKAGES").split(),
            "supported_devices": getenv("SUPPORTED_DEVICES").split(),
            "titles": get_titles(),
        }
    },
}

if getenv("FILE_FILESYSTEM"):
    file_info["profiles"][device_id]["images"][0]["filesystem"] = getenv(
        "FILE_FILESYSTEM"
    )

json_path.write_text(json.dumps(file_info, separators=(",", ":")))



#!/usr/bin/python3
# SPDX-License-Identifier: GPL-2.0-or-later
#
# Copyright (C) 2022 Luiz Angelo Daros de Luca <luizluca@gmail.com>
#
# Cameo Image header geneator, used by some D-Link DGS-1210 switches
# and APRESIA ApresiaLightGS series
#
import argparse
import pathlib
import socket
import struct

MODEL_LEN = 20
SIGNATURE_LEN = 16
LINUXLOAD_LEN = 10
BUFSIZE = 4096

parser = argparse.ArgumentParser(description='Generate Cameo firmware header.')
parser.add_argument('source_file', type=argparse.FileType('rb'))
parser.add_argument('dest_file', type=argparse.FileType('wb'))
parser.add_argument('model')
parser.add_argument('signature')
parser.add_argument('partition', type=int, choices=range(0,10),
                    metavar="partition=[0-9]",help="partition id")
parser.add_argument('customer_signature', type=int, choices=range(0,10),
                    metavar="customer_signature=[0-9]",
                    help="customer signature")
parser.add_argument('board_version', type=int, choices=range(0,2**32),
                    metavar="board_version=[0-4294967295]",
                    help="board version")
parser.add_argument('linux_loadaddr', nargs='?',
                    help="Kernel start address in 0xFFFFFFFF format")
args = parser.parse_args()

if len(args.model) > MODEL_LEN:
    raise ValueError(f"Model '{args.model}' is greater than {MODEL_LEN} bytes")

if len(args.signature) > SIGNATURE_LEN:
    raise ValueError(f"Signature '{args.signature}' is greater than"
                     f"{SIGNATURE_LEN} bytes")

if args.signature == "os":
    if args.linux_loadaddr:
        if len(args.linux_loadaddr) > LINUXLOAD_LEN:
            raise ValueError(f"linux_loadaddr '{args.linux_loadaddr}' is greater"
                             f"than {LINUXLOAD_LEN} bytes")
        if (args.linux_loadaddr[0:2] != "0x"):
            raise ValueError(f"linux_loadaddr '{args.linux_loadaddr}' must use"
                             f"the 0x789ABCDE format")
        int(args.linux_loadaddr[2:],16)
    else:
        raise ValueError(f"linux_loadaddr is required for signature 'os'")
else:
    args.linux_loadaddr = ""

checksum = 0
size = 0
while True:
    buf = args.source_file.read(BUFSIZE)
    if not buf:
        break
    checksum = sum(iter(buf),checksum) % (1<<32)
    size += len(buf)

args.dest_file.write(struct.pack('!I', checksum))
args.dest_file.write(struct.pack(f'{MODEL_LEN}s',
                                 args.model.encode("ascii")))
args.dest_file.write(struct.pack(f'{SIGNATURE_LEN}s',
                                 args.signature.encode("ascii")))
args.dest_file.write(struct.pack('!B', args.partition))
args.dest_file.write(struct.pack('!B', 0x40)) # ??? This header size?
args.dest_file.write(struct.pack('!B', 0x00)) # ??? Encrypted?
args.dest_file.write(struct.pack('!B', args.customer_signature))
args.dest_file.write(struct.pack('!I', args.board_version))
args.dest_file.write(struct.pack('!I', size))
args.dest_file.write(struct.pack(f'{LINUXLOAD_LEN}s',
                                 args.linux_loadaddr.encode("ascii")))
args.dest_file.write(struct.pack('!2x'))

args.source_file.seek(0)
while True:
    buf = args.source_file.read(BUFSIZE)
    if not buf:
        break
    args.dest_file.write(buf)



#!/usr/bin/env python3
"""
# SPDX-License-Identifier: GPL-2.0-or-later
#
# sercomm-kernel-header.py: Creates Sercomm kernel header
#
# Copyright © 2022 Mikhail Zhilkin
"""

import argparse
import binascii
import os
import struct

KERNEL_HEADER_SIZE = 0x100
PADDING = 0xff
ROOTFS_FAKE_HEADER = "UBI#"

def auto_int(x):
	return int(x, 0)

def create_kernel_header(args):
	out_file = open(args.header_file, "wb")
	header = get_kernel_header(args)
	out_file.write(header)
	out_file.close()

def get_kernel_header(args):
	header = bytearray([PADDING] * KERNEL_HEADER_SIZE)

	struct.pack_into('<L', header, 0xc, 0xffffff02)
	struct.pack_into('<L', header, 0x1c, 0x0)
	struct.pack_into('<L', header, 0x34, 0x0)
	struct.pack_into('<L', header, 0x10, args.kernel_offset)
	struct.pack_into('<L', header, 0x28, args.rootfs_offset)

	if (args.rootfs_file):
		if (args.rootfs_checking_size):
			rootfs_size = args.rootfs_checking_size
		else:
			rootfs_size = os.path.getsize(args.rootfs_file)
		buf = open(args.rootfs_file,'rb').read(rootfs_size)
		crc = binascii.crc32(buf) & 0xffffffff
	else:
		rootfs_size = len(ROOTFS_FAKE_HEADER)
		crc = binascii.crc32(str.encode(ROOTFS_FAKE_HEADER)) & \
		      0xffffffff
	struct.pack_into('<L', header, 0x2c, rootfs_size)
	struct.pack_into('<L', header, 0x30, crc)

	kernel_size = os.path.getsize(args.kernel_file)
	struct.pack_into('<L', header, 0x14, kernel_size)

	kernel_end_offset = args.kernel_offset + kernel_size
	struct.pack_into('<L', header, 0x4, kernel_end_offset)

	buf = open(args.kernel_file,'rb').read()
	crc = binascii.crc32(buf) & 0xffffffff
	struct.pack_into('<L', header, 0x18, crc)

	crc = binascii.crc32(header) & 0xffffffff
	struct.pack_into('<L', header, 0x8, crc)

	struct.pack_into('<L', header, 0x0, 0x726553)

	return header

def main():
	global args

	parser = argparse.ArgumentParser(description='This script generates \
		a kernel header for the Sercomm mt7621 devices')

	parser.add_argument('--kernel-image',
		dest='kernel_file',
		action='store',
		type=str,
		help='Path to a Kernel binary image')

	parser.add_argument('--kernel-offset',
		dest='kernel_offset',
		action='store',
		type=auto_int,
		help='Kernel offset')

	parser.add_argument('--rootfs-offset',
		dest='rootfs_offset',
		action='store',
		type=auto_int,
		help='RootFS offset')

	parser.add_argument('--output-header',
		dest='header_file',
		action='store',
		type=str,
		help='Output kernel header file')

	parser.add_argument('--rootfs-image',
		dest='rootfs_file',
		action='store',
		type=str,
		help='Path to RootFS binary image (optional)')

	parser.add_argument('--rootfs-checking-size',
		dest='rootfs_checking_size',
		action='store',
		type=auto_int,
		help='Bytes count for CRC calculation (optional)')

	args = parser.parse_args()

	if ((not args.kernel_file) or
	    (not args.kernel_offset) or
	    (not args.rootfs_offset) or
	    (not args.header_file)):
		parser.print_help()
		exit()

	create_kernel_header(args)

main()



#!/usr/bin/python3
# SPDX-License-Identifier: GPL-2.0-or-later
#
# Copyright (C) 2024 OpenWrt.org
#
# ./belkin-header.py <ImageFileIn> <ImageFileOut> <BelkinHeader> <BelkinModel>
#
# This script adds an image header for Belkin devices. As of now only Realtek
# based switches of the Linksys LGS3xxC/LGS3xxMPC series are known to use this
# format. It resembles a U-Boot legacy format image header, all data in network
# byte order (aka natural aka big endian).
#
# Known values for BelkinHeader are
#
# 0x07800001 : RTL838x based switch
# 0x07600001 : RTL93xx based switch
#
# Known values for BelkinModel are
#
# BKS-RTL83xx : RTL838x based switch
# BKS-RTL93xx : RTL93xx based switch

import argparse
import os
import zlib
import array
import sys
import time

VERSION1 = 1
VERSION2 = 1
VERSION3 = 2
VERSION4 = 2
COMPANY = "belkin"
MODULE = "IMG"

def xcrc32(buf):
    return (0xffffffff - zlib.crc32(buf, 0xffffffff)).to_bytes(4, byteorder='big')

def encode_model(model):
    map = " 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz-"
    code = bytearray()

    code.append(map.index(model[:1]))
    model = model[1:]
    model = model + " " * (3 - len(model) % 4)

    while model != "":
        b1 = map.index(model[0:1])
        b2 = map.index(model[1:2])
        b3 = map.index(model[2:3])
        b4 = map.index(model[3:4])
        model = model[4:]

        code.append(b1 << 2 | b2 >> 4)
        code.append((b2 & 0xf) << 4 | b3 >> 2)
        code.append((b3 & 0x3) << 6 | b4)

    return code

def create_header(buf, belkin_header, belkin_model):
    head = bytearray(32)

    head[0:4] = int(belkin_header, 0).to_bytes(4, 'big')
    head[8:12] = int(time.time()).to_bytes(4, 'big')
    head[12:16] = len(buf).to_bytes(4, byteorder='big')
    head[24:28] = xcrc32(buf)
    head[28:29] = VERSION1.to_bytes(1, byteorder='big')
    head[29:30] = VERSION2.to_bytes(1, byteorder='big')
    head[30:31] = VERSION3.to_bytes(1, byteorder='big')
    head[31:32] = VERSION4.to_bytes(1, byteorder='big')
    head[16:16 + len(COMPANY)] = bytes(COMPANY,'ascii')

    mod = MODULE + "-{:1d}.{:02d}.{:02d}.{:02d}".format(VERSION1, VERSION2, VERSION3, VERSION4)
    head.extend(bytes(mod,'ascii'))
    head.append(0x00)
    head.extend(encode_model(belkin_model))
    head.extend(bytes([0x00] * (64 - len(head))))

    head[4:8] = xcrc32(head)

    return head

parser = argparse.ArgumentParser(description='Generate Belkin header.')
parser.add_argument('source', type=argparse.FileType('r+b'))
parser.add_argument('dest', type=argparse.FileType('wb'))
parser.add_argument('belkin_header')
parser.add_argument('belkin_model')
args = parser.parse_args()

buf = bytearray(args.source.read())
head = create_header(buf, args.belkin_header, args.belkin_model)
args.dest.write(head)
args.dest.write(buf)



#!/usr/bin/env python3
#
# Copyright (c) 2018 Yousong Zhou <yszhou4tech@gmail.com>
#
# This is free software, licensed under the GNU General Public License v2.
# See /LICENSE for more information.

import argparse
import calendar
import datetime
import errno
import fcntl
import hashlib
import json
import os
import os.path
import re
import shutil
import ssl
import subprocess
import sys
import time
import urllib.request

TMPDIR = os.environ.get('TMP_DIR') or '/tmp'
TMPDIR_DL = os.path.join(TMPDIR, 'dl')


class PathException(Exception): pass
class DownloadGitHubError(Exception): pass


class Path(object):
    """Context class for preparing and cleaning up directories.

    If ```preclean` is ``False``, ``path`` will NOT be removed on context enter

    If ``path`` ``isdir``, then it will be created on context enter.

    If ``keep`` is True, then ``path`` will NOT be removed on context exit
    """

    def __init__(self, path, isdir=True, preclean=False, keep=False):
        self.path = path
        self.isdir = isdir
        self.preclean = preclean
        self.keep = keep

    def __enter__(self):
        if self.preclean:
            self.rm_all(self.path)
        if self.isdir:
            self.mkdir_all(self.path)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if not self.keep:
            self.rm_all(self.path)

    @staticmethod
    def mkdir_all(path):
        """Same as mkdir -p."""
        names = os.path.split(path)
        p = ''
        for name in names:
            p = os.path.join(p, name)
            Path._mkdir(p)

    @staticmethod
    def _rmdir_dir(dir_):
        names = Path._listdir(dir_)
        for name in names:
            p = os.path.join(dir_, name)
            Path.rm_all(p)
        Path._rmdir(dir_)

    @staticmethod
    def _mkdir(path):
        Path._os_func(os.mkdir, path, errno.EEXIST)

    @staticmethod
    def _rmdir(path):
        Path._os_func(os.rmdir, path, errno.ENOENT)

    @staticmethod
    def _remove(path):
        Path._os_func(os.remove, path, errno.ENOENT)

    @staticmethod
    def _listdir(path):
        return Path._os_func(os.listdir, path, errno.ENOENT, default=[])

    @staticmethod
    def _os_func(func, path, errno, default=None):
        """Call func(path) in an idempotent way.

        On exception ``ex``, if the type is OSError and ``ex.errno == errno``,
        return ``default``, otherwise, re-raise
        """
        try:
            return func(path)
        except OSError as e:
            if e.errno == errno:
                return default
            else:
                raise

    @staticmethod
    def rm_all(path):
        """Same as rm -r."""
        if os.path.islink(path):
            Path._remove(path)
        elif os.path.isdir(path):
            Path._rmdir_dir(path)
        else:
            Path._remove(path)

    @staticmethod
    def untar(path, into=None):
        """Extract tarball at ``path`` into subdir ``into``.

        return subdir name if and only if there exists one, otherwise raise PathException
        """
        args = ('tar', '-C', into, '-xzf', path, '--no-same-permissions')
        subprocess.check_call(args, preexec_fn=lambda: os.umask(0o22))
        dirs = os.listdir(into)
        if len(dirs) == 1:
            return dirs[0]
        else:
            raise PathException('untar %s: expecting a single subdir, got %s' % (path, dirs))

    @staticmethod
    def tar(path, subdir, into=None, ts=None):
        """Pack ``path`` into tarball ``into``."""
        # --sort=name requires a recent build of GNU tar
        args = ['tar', '--numeric-owner', '--owner=0', '--group=0', '--sort=name', '--mode=a-s']
        args += ['-C', path, '-cf', into, subdir]
        envs = os.environ.copy()
        if ts is not None:
            args.append('--mtime=@%d' % ts)
        if into.endswith('.zst'):
            args.append('-I zstd -T0 --ultra -20')
        elif into.endswith('.xz'):
            envs['XZ_OPT'] = '-7e'
            args.append('-J')
        elif into.endswith('.bz2'):
            args.append('-j')
        elif into.endswith('.gz'):
            args.append('-z')
            envs['GZIP'] = '-n'
        else:
            raise PathException('unknown compression type %s' % into)
        subprocess.check_call(args, env=envs)


class GitHubCommitTsCache(object):
    __cachef = 'github.commit.ts.cache'
    __cachen = 2048

    def __init__(self):
        Path.mkdir_all(TMPDIR_DL)
        self.cachef = os.path.join(TMPDIR_DL, self.__cachef)
        self.cache = {}

    def get(self, k):
        """Get timestamp with key ``k``."""
        fileno = os.open(self.cachef, os.O_RDONLY | os.O_CREAT)
        with os.fdopen(fileno) as fin:
            try:
                fcntl.lockf(fileno, fcntl.LOCK_SH)
                self._cache_init(fin)
                if k in self.cache:
                    ts = self.cache[k][0]
                    return ts
            finally:
                fcntl.lockf(fileno, fcntl.LOCK_UN)
        return None

    def set(self, k, v):
        """Update timestamp with ``k``."""
        fileno = os.open(self.cachef, os.O_RDWR | os.O_CREAT)
        with os.fdopen(fileno, 'w+') as f:
            try:
                fcntl.lockf(fileno, fcntl.LOCK_EX)
                self._cache_init(f)
                self.cache[k] = (v, int(time.time()))
                self._cache_flush(f)
            finally:
                fcntl.lockf(fileno, fcntl.LOCK_UN)

    def _cache_init(self, fin):
        for line in fin:
            k, ts, updated = line.split()
            ts = int(ts)
            updated = int(updated)
            self.cache[k] = (ts, updated)

    def _cache_flush(self, fout):
        cache = sorted(self.cache.items(), key=lambda a: a[1][1])
        cache = cache[:self.__cachen]
        self.cache = {}
        os.ftruncate(fout.fileno(), 0)
        fout.seek(0, os.SEEK_SET)
        for k, ent in cache:
            ts = ent[0]
            updated = ent[1]
            line = '{0} {1} {2}\n'.format(k, ts, updated)
            fout.write(line)


class DownloadGitHubTarball(object):
    """Download and repack archive tarball from GitHub.

    Compared with the method of packing after cloning the whole repo, this
    method is more friendly to users with fragile internet connection.

    However, there are limitations with this method

     - GitHub imposes a 60 reqs/hour limit for unauthenticated API access.
       This affects fetching commit date for reproducible tarballs.  Download
       through the archive link is not affected.

     - GitHub archives do not contain source codes for submodules.

     - GitHub archives seem to respect .gitattributes and ignore paths with
       export-ignore attributes.

    For the first two issues, the method will fail loudly to allow fallback to
    clone-then-pack method.

    As for the 3rd issue, to make sure that this method only produces identical
    tarballs as the fallback method, we require the expected hash value to be
    supplied.  That means the first tarball will need to be prepared by the
    clone-then-pack method
    """

    __repo_url_regex = re.compile(r'^(?:https|git)://github.com/(?P<owner>[^/]+)/(?P<repo>[^/]+)')

    def __init__(self, args):
        self.dl_dir = args.dl_dir
        self.version = args.version
        self.subdir = args.subdir
        self.source = args.source
        self.submodules = args.submodules
        self.url = args.url
        self._init_owner_repo()
        self.xhash = args.hash
        self._init_hasher()
        self.commit_ts = None           # lazy load commit timestamp
        self.commit_ts_cache = GitHubCommitTsCache()
        self.name = 'github-tarball'

    def download(self):
        """Download and repack GitHub archive tarball."""
        if self.submodules and self.submodules != ['skip']:
            raise self._error('Fetching submodules is not yet supported')
        self._init_commit_ts()
        with Path(TMPDIR_DL, keep=True) as dir_dl:
            # fetch tarball from GitHub
            tarball_path = os.path.join(dir_dl.path, self.subdir + '.tar.gz.dl')
            with Path(tarball_path, isdir=False):
                self._fetch(tarball_path)
                # unpack
                d = os.path.join(dir_dl.path, self.subdir + '.untar')
                with Path(d, preclean=True) as dir_untar:
                    tarball_prefix = Path.untar(tarball_path, into=dir_untar.path)
                    dir0 = os.path.join(dir_untar.path, tarball_prefix)
                    dir1 = os.path.join(dir_untar.path, self.subdir)
                    # submodules check
                    if self.submodules != ['skip'] and self._has_submodule(dir0):
                        raise self._error('Fetching submodules is not yet supported')
                    # rename subdir
                    os.rename(dir0, dir1)
                    # repack
                    into=os.path.join(TMPDIR_DL, self.source)
                    Path.tar(dir_untar.path, self.subdir, into=into, ts=self.commit_ts)
                    try:
                        self._hash_check(into)
                    except Exception:
                        Path.rm_all(into)
                        raise
                    # move to target location
                    file1 = os.path.join(self.dl_dir, self.source)
                    if into != file1:
                        shutil.move(into, file1)

    def _has_submodule(self, dir_):
        m = os.path.join(dir_, '.gitmodules')
        try:
            st = os.stat(m)
            return st.st_size > 0
        except OSError as e:
            return e.errno != errno.ENOENT

    def _init_owner_repo(self):
        m = self.__repo_url_regex.search(self.url)
        if m is None:
            raise self._error('Invalid github url: {}'.format(self.url))
        owner = m.group('owner')
        repo = m.group('repo')
        if repo.endswith('.git'):
            repo = repo[:-4]
        self.owner = owner
        self.repo = repo

    def _init_hasher(self):
        xhash = self.xhash
        if len(xhash) == 64:
            self.hasher = hashlib.sha256()
        elif len(xhash) == 32:
            self.hasher = hashlib.md5()
        else:
            raise self._error('Requires sha256sum for verification')
        self.xhash = xhash

    def _hash_check(self, f):
        with open(f, 'rb') as fin:
            while True:
                d = fin.read(4096)
                if not d:
                    break
                self.hasher.update(d)
        xhash = self.hasher.hexdigest()
        if xhash != self.xhash:
            raise self._error('Wrong hash (probably caused by .gitattributes), expecting {}, got {}'.format(self.xhash, xhash))

    def _init_commit_ts(self):
        if self.commit_ts is not None:
            return
        # GitHub provides 2 APIs[1,2] for fetching commit data.  API[1] is more
        # terse while API[2] provides more verbose info such as commit diff
        # etc.  That's the main reason why API[1] is preferred: the response
        # size is predictable.
        #
        # However, API[1] only accepts complete commit sha1sum as the parameter
        # while API[2] is more liberal accepting also partial commit id and
        # tags, etc.
        #
        # [1] Get a single commit, Repositories, https://developer.github.com/v3/repos/commits/#get-a-single-commit
        # [2] Git Commits, Git Data, https://developer.github.com/v3/git/commits/#get-a-commit
        apis = [
            {
                'url': self._make_repo_url_path('git', 'commits', self.version),
                'attr_path': ('committer', 'date'),
            }, {
                'url': self._make_repo_url_path('commits', self.version),
                'attr_path': ('commit', 'committer', 'date'),
            },
        ]
        version_is_sha1sum = len(self.version) == 40
        if not version_is_sha1sum:
            apis.insert(0, apis.pop())
        reasons = ''
        for api in apis:
            url = api['url']
            attr_path = api['attr_path']
            try:
                ct = self.commit_ts_cache.get(url)
                if ct is not None:
                    self.commit_ts = ct
                    return
                ct = self._init_commit_ts_remote_get(url, attr_path)
                self.commit_ts = ct
                self.commit_ts_cache.set(url, ct)
                return
            except Exception as e:
                reasons += '\n' + ("  {}: {}".format(url, e))
        raise self._error('Cannot fetch commit ts:{}'.format(reasons))

    def _init_commit_ts_remote_get(self, url, attrpath):
        resp = self._make_request(url)
        data = resp.read()
        date = json.loads(data)
        for attr in attrpath:
            date = date[attr]
        date = datetime.datetime.strptime(date, '%Y-%m-%dT%H:%M:%SZ')
        date = date.timetuple()
        ct = calendar.timegm(date)
        return ct

    def _fetch(self, path):
        """Fetch tarball of the specified version ref."""
        ref = self.version
        url = self._make_repo_url_path('tarball', ref)
        resp = self._make_request(url)
        with open(path, 'wb') as fout:
            while True:
                d = resp.read(4096)
                if not d:
                    break
                fout.write(d)

    def _make_repo_url_path(self, *args):
        url = '/repos/{0}/{1}'.format(self.owner, self.repo)
        if args:
            url += '/' + '/'.join(args)
        return url

    def _make_request(self, path):
        """Request GitHub API endpoint on ``path``."""
        url = 'https://api.github.com' + path
        headers = {
            'Accept': 'application/vnd.github.v3+json',
            'User-Agent': 'OpenWrt',
        }
        req = urllib.request.Request(url, headers=headers)
        sslcontext = ssl._create_unverified_context()
        fileobj = urllib.request.urlopen(req, context=sslcontext)
        return fileobj

    def _error(self, msg):
        return DownloadGitHubError('{}: {}'.format(self.source, msg))


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--dl-dir', default=os.getcwd(), help='Download dir')
    parser.add_argument('--url', help='Download URL')
    parser.add_argument('--subdir', help='Source code subdir name')
    parser.add_argument('--version', help='Source code version')
    parser.add_argument('--source', help='Source tarball filename')
    parser.add_argument('--hash', help='Source tarball\'s expected sha256sum')
    parser.add_argument('--submodules', nargs='*', help='List of submodules, or "skip"')
    args = parser.parse_args()
    try:
        method = DownloadGitHubTarball(args)
        method.download()
    except Exception as ex:
        sys.stderr.write('{}: Download from {} failed\n'.format(args.source, args.url))
        sys.stderr.write('{}\n'.format(ex))
        sys.exit(1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import argparse

from ftplib import FTP
from os import stat

parser = argparse.ArgumentParser(description='Tool to boot AVM EVA ramdisk images.')
parser.add_argument('ip', type=str, help='IP-address to transfer the image to')
parser.add_argument('image', type=str, help='Location of the ramdisk image')
parser.add_argument('--offset', type=lambda x: int(x,0), help='Offset to load the image to in hex format with leading 0x. Only needed for non-lantiq devices.')
args = parser.parse_args()

size = stat(args.image).st_size
# arbitrary size limit, to prevent the address calculations from overflows etc.
assert size < 0x2000000

if args.offset:
	addr = size
	haddr = args.offset
else:
	# We need to align the address.
	# A page boundary seems to be sufficient on 7362sl and 7412
	addr = ((0x8000000 - size) & ~0xfff)
	haddr = 0x80000000 + addr

img = open(args.image, "rb")
ftp = FTP(args.ip, 'adam2', 'adam2')

def adam(cmd):
	print("> %s"%(cmd))
	resp = ftp.sendcmd(cmd)
	print("< %s"%(resp))
	assert resp[0:3] == "200"

ftp.set_pasv(True)
# The following parameters allow booting the avm recovery system with this
# script.
adam('SETENV memsize 0x%08x'%(addr))
adam('SETENV kernel_args_tmp mtdram1=0x%08x,0x88000000'%(haddr))
adam('MEDIA SDRAM')
ftp.storbinary('STOR 0x%08x 0x88000000'%(haddr), img)
img.close()
ftp.close()



#!/usr/bin/env python3
#
# Copyright 2008, 2009 (C) Jose Vasconcellos <jvasco@verizon.net>
#
# A script that can communicate with jungo-based routers
# (such as MI424-WR, USR8200 and WRV54G) to backup the installed
# firmware and replace the boot loader.
#
# Tested with Python 2.5 on Linux and Windows
#
"""Usage: %s [options] <IP_address> [image.bin | url]
Valid options:
\t-h | --help: usage statement
\t-d | --dump: create a flash dump
\t-f | --file: use <filename> to store dump contents
\t-u | --user: provide username (default admin)
\t-p | --pass: provide password (default password1)
\t     --port: set port for http (default 8080)
\t-q | --quiet: don't display unnecessary information
\t-r | --reboot: reboot target on successful transfer
\t-V | --version: display version information

If no image (or url) is given, a flash dump is created.
A built-in http server is used when an image file is provided.
"""

import os
import sys
import getopt
import getpass
import telnetlib
import string
import binascii
import socket
import _thread
import socketserver
import http.server

reboot = 0
HOST = "192.168.1.1"
PORT = 8080
user = "admin"
#password = getpass.getpass()
password = "password1"
proto = "http"
url = ""
imagefile = ""
dumpfile = ""
verbose = 1
do_dump = 0
dumplen = 0x10000
flashsize=4*1024*1024
#device="br0"
device="ixp0"

####################

def start_server(server):
    httpd = socketserver.TCPServer((server,PORT),http.server.SimpleHTTPRequestHandler)
    _thread.start_new_thread(httpd.serve_forever,())

####################

def get_flash_size():
    # make sure we don't have an A0 stepping
    tn.write("cat /proc/cpuinfo\n")
    buf = tn.read_until("Returned 0", 3)
    if not buf:
        print("Unable to obtain CPU information; make sure to not use A0 stepping!")
    elif buf.find('rev 0') > 0:
        print("Warning: IXP42x stepping A0 detected!")
        if imagefile or url:
            print("Error: No linux support for A0 stepping!")
            sys.exit(2)

    # now get flash size
    tn.write("cat /proc/mtd\n")
    buf = tn.read_until("Returned 0", 3)
    if buf:
        i = buf.find('mtd0:')
        if i > 0:
            return int(buf[i+6:].split()[0],16)
        # use different command
        tn.write("flash_layout\n")
        buf = tn.read_until("Returned 0", 3)
        i = buf.rfind('Range ')
        if i > 0:
            return int(buf[i+17:].split()[0],16)
        print("Can't determine flash size!")
    else:
        print("Unable to obtain flash size!")
    sys.exit(2)

def image_dump(tn, dumpfile):
    if not dumpfile:
        tn.write("ver\n");
        buf = tn.read_until("Returned 0",2)
        i = buf.find("Platform:")
        if i < 0:
	    platform="jungo"
	else:
	    line=buf[i+9:]
	    i=line.find('\n')
	    platform=line[:i].split()[-1]

        tn.write("rg_conf_print /dev/%s/mac\n" % device);
        buf = tn.read_until("Returned 0",3)

	i = buf.find("mac(")
	if i > 0:
	    i += 4
	else:
	    print("No MAC address found! (use -f option)")
	    sys.exit(1)
        dumpfile = "%s-%s.bin" % (platform, buf[i:i+17].replace(':',''))
    else:
        tn.write("\n")

    print("Dumping flash contents (%dMB) to %s" % (flashsize/1048576, dumpfile))
    f = open(dumpfile, "wb")

    t=flashsize/dumplen
    for addr in range(t):
	if verbose:
	    sys.stdout.write('\r%d%%'%(100*addr/t))
	    sys.stdout.flush()

        tn.write("flash_dump -r 0x%x -l %d -4\n" % (addr*dumplen, dumplen))
	tn.read_until("\n")

	count = addr*dumplen
        while 1:
            buf = tn.read_until("\n")
            if buf.strip() == "Returned 0":
                break
            s = buf.split()
            if s and s[0][-1] == ':':
		a=int(s[0][:-1],16)
		if a != count:
		    print("Format error: %x != %x"%(a,count))
		    sys.exit(2)
	    	count += 16
		f.write(binascii.a2b_hex(string.join(s[1:],'')))
	tn.read_until(">",1)

    f.close()
    if verbose:
	print("")

def telnet_option(sock,cmd,option):
    #print "Option: %d %d" % (ord(cmd), ord(option))
    if cmd == telnetlib.DO:
        c=telnetlib.WILL
    elif cmd == telnetlib.WILL:
        c=telnetlib.DO
    sock.sendall(telnetlib.IAC + c + option)

def telnet_timeout():
    print("Fatal error: telnet timeout!")
    sys.exit(1)

def usage():
    print(__doc__ % os.path.basename(sys.argv[0]))

####################

try:
    opts, args = getopt.getopt(sys.argv[1:], "hdf:qp:P:rvV", \
	["help", "dump", "file=", "user=", "pass=", "port=",
	 "quiet=", "reboot", "verbose", "version"])
except getopt.GetoptError:
    # print help information and exit:
    usage()
    sys.exit(1)

for o, a in opts:
    if o in ("-h", "--help"):
	usage()
	sys.exit(1)
    elif o in ("-V", "--version"):
	print("%s: 0.11" % sys.argv[0])
	sys.exit(1)
    elif o in ("-d", "--no-dump"):
	do_dump = 1
    elif o in ("-f", "--file"):
	dumpfile = a
    elif o in ("-u", "--user"):
	user = a
    elif o in ("-p", "--pass"):
	password = a
    elif o == "--port":
	PORT = int(a)
    elif o in ("-q", "--quiet"):
	verbose = 0
    elif o in ("-r", "--reboot"):
	reboot = 1
    elif o in ("-v", "--verbose"):
	verbose = 1

# make sure we have enough arguments
if len(args) > 0:
    HOST = args[0]

if len(args) == 2:
    if args[1].split(':')[0] in ("tftp", "http", "ftp"):
        url = args[1]
    else:
        imagefile = args[1]
else:
    do_dump = 1;

####################
# create a telnet session to the router
try:
    tn = telnetlib.Telnet(HOST)
except socket.error as msg:
    print("Unable to establish telnet session to %s: %s" % (HOST, msg))
    sys.exit(1)

tn.set_option_negotiation_callback(telnet_option)

buf = tn.read_until("Username: ", 3)
if not buf:
    telnet_timeout()
tn.write(user+"\n")
if password:
    buf = tn.read_until("Password: ", 3)
    if not buf:
        telnet_timeout()
    tn.write(password+"\n")

# wait for prompt
buf = tn.read_until("> ", 3)
if not buf:
    telnet_timeout()

flashsize = get_flash_size()

if do_dump:
    image_dump(tn, dumpfile)

if imagefile or url:
    splitpath = os.path.split(imagefile)

    # create load command
    if url:
        cmd = "load -u %s -r 0\n" % (url)
    else:
        server = tn.get_socket().getsockname()[0]
        cmd = "load -u http://%s:%d/%s -r 0\n" % (server, PORT, splitpath[1])

        if not os.access(imagefile, os.R_OK):
            print("File access error: %s" % (imagefile))
            sys.exit(3)

        # make sure we're in the directory where the image is located
        if splitpath[0]:
            os.chdir(splitpath[0])

        start_server(server)

    if verbose:
	print("Unlocking flash...")
    tn.write("unlock 0 0x%x\n" % flashsize)
    buf = tn.read_until("Returned 0",5)

    if verbose:
	print("Writing new image...")
    print(cmd, end=' ')
    tn.write(cmd)
    buf = tn.read_until("Returned 0",10)

    # wait till the transfer completed
    buf = tn.read_until("Download completed successfully",20)
    if buf:
	print("Flash update complete!")
        if reboot:
            tn.write("reboot\n")
            print("Rebooting...")

tn.write("exit\n")
tn.close()

